import json
import sys
import time
import requests
from kubernetes import client, config
from kazoo.client import KazooClient

# APP_LABEL_K8S = sys.argv[1]
# NAMESPACE = sys.argv[2]

def zk_status_check(namespace, app_label):
    statefulset = api_instance.list_namespaced_stateful_set(
        namespace, watch=False, label_selector='app=' + app_label)
    if not statefulset:
        print("stateful set for %s not found" % app_label)
        return False
    try:
        if not statefulset.items[0].status.updated_replicas:
            print("updated replica count is not yet available")
            return False
    except Exception as e:
        print(e)
        print("update replica count is not yet available")
        return  False

    # FETCH DESIRED NUMBER OF REPLICA COUNT
    replicas = statefulset.items[0].status.replicas

    # VERIFICATION OF UNDER REPLICATED REPLICAS
    if statefulset.items[0].status.updated_replicas != replicas:
        print("updated replicas are %s and replica count is %s" %(statefulset.items[0].status.updated_replicas, replicas))
        return False
    # VERIFICATION OF NON-READY REPLICAS
    if statefulset.items[0].status.ready_replicas != replicas:
        print("ready replicas are %s and replica count is %s" % (statefulset.items[0].status.ready_replicas, replicas))
        return False

    # VERIFICATION OF POD REACHABILITY THROUGH JOLOKIA ENDPOINT
    for pod_id in range(0,replicas):
      try:
        zk_pod_id = "{}-{}".format(app_label,str(pod_id))
        zk_pod_check_endpoint = "http://{}.{}-headless.{}.svc.cluster.local:8778/jolokia/read/jolokia:type=Discovery".format(zk_pod_id,app_label,namespace)
        print("trying to reach zk pod at {}".format(zk_pod_check_endpoint))
        response = requests.get(zk_pod_check_endpoint)
        if response.status_code != 200:
          print("zk pod unreachable at {}".format(zk_pod_check_endpoint))
          return False
        print("reached zk pod at {}".format(zk_pod_check_endpoint))
      except Exception as e:
        print(e)
        print("zk pod is not reachable")
        return False

    # VERIFICATION OF KAFKA CLUSTER REGISTRATION FROM KAFKA CLIENT
    zk_cluster_endpoint = "{}-client.{}.svc.cluster.local:2181".format(app_label,namespace)
    try:
        print("checking cluster node in zookeeper")
        zk = KazooClient(zk_cluster_endpoint)
        zk.start()
        cluster_exists = zk.exists('/KafkaCluster1')
        if cluster_exists is None:
            print("cluster node is not present ")
            return False
        print("cluster node is present")
        zk.stop()
    except Exception as e:
        print(e)
        print("error during cluster node verification")
        return False

    print("Replicas are updated and ready to serve")
    return True


def is_zk_cluster_ready(namespace, app_label):
    time_counter = 10
    while time_counter:
        result = zk_status_check(namespace, app_label)
        if result:
            return True
        time_counter = time_counter - 1
        time.sleep(30)
        print("remaining retries %s" % time_counter)
    return False


if __name__ == "__main__":
    print("Temporary fix to bypass zk-cluster-status-check")
    # config.load_incluster_config()
    # api_instance = client.AppsV1Api()
    # if not is_zk_cluster_ready(NAMESPACE, APP_LABEL_K8S):
    #     raise Exception("Zk Cluster is not ready.")